; bassclar.orc

; instr 15 - bass clarinet
; instr 194 - reverb
sr=32000
kr=1600
ksmps=20
nchnls=2

giseed = .5
giwtsin = 1
garev   init    0
;--------------------------------------------------------------------- 

instr 15                                                ; bass clarinet

;       parameters
;       p4      overall amplitude scaling factor
;       p5      pitch in Hertz (normal pitch range: sounding Gb1 - Db4)
;               bass clarinet range from Db2 - Db4, contrabass clarinet from Gb1 - Gb3
;                       (break bewteen C2 and Db2)
;       p6      percent vibrato depth, recommended values in range [-1., +1.]
;                       0.0     -> no vibrato
;                       +1.     -> 1% vibrato depth, where vibrato rate increases slightly
;                       -1.     -> 1% vibrato depth, where vibrato rate decreases slightly
;       p7      attack time in seconds 
;                       recommended value:  .06 for tongued notes (.03 for short notes)
;       p8      decay time in seconds 
;                       recommended value:  .15 (.04 for short notes)
;       p9      overall brightness / filter cutoff factor 

;                       1 -> least bright / minimum filter cutoff frequency (40 Hz)
;                       9 -> brightest / maximum filter cutoff frequency (10,240Hz)

                                                        ; initial variables
iampscale       =       p4                              ; overall amplitude scaling factor
ifreq           =       p5                              ; pitch in Hertz
ivibdepth       =       abs(p6*ifreq/100.0)             ; vibrato depth relative to fundamental frequency
iattack         =       p7 * (1.1 - .2*giseed)          ; attack time with up to +-10% random deviation
giseed          =       frac(giseed*105.947)            ; reset giseed
idecay          =       p8 * (1.1 - .2*giseed)          ; decay time with up to +-10% random deviation
giseed          =       frac(giseed*105.947)
ifiltcut        tablei  p9, 2                           ; lowpass filter cutoff frequency

iattack         =       (iattack < 6/kr ? 6/kr : iattack)               ; minimal attack length
idecay          =       (idecay < 6/kr ? 6/kr : idecay)                 ; minimal decay length
isustain        =       p3 - iattack - idecay
p3              =       (isustain < 5/kr ? iattack+idecay+5/kr : p3)    ; minimal sustain length
isustain        =       (isustain < 5/kr ? 5/kr : isustain)                     
iatt            =       iattack/6
isus            =       isustain/4
idec            =       idecay/6
iphase          =       giseed                          ; use same phase for all wavetables
giseed          =       frac(giseed*105.947)

                                                        ; vibrato block
kvibdepth       linseg  .1, .8*p3, 1, .2*p3, .7
kvibdepth       =       kvibdepth* ivibdepth            ; vibrato depth
kvibdepthr      randi   .1*kvibdepth, 5, giseed         ; up to 10% vibrato depth variation
giseed          =       frac(giseed*105.947)
kvibdepth       =       kvibdepth + kvibdepthr
ivibr1          =       giseed                          ; vibrato rate
giseed          =       frac(giseed*105.947)
ivibr2          =       giseed
giseed          =       frac(giseed*105.947)

if p6 < 0 goto vibrato1
kvibrate        linseg  2.5+ivibr1, p3, 4.5+ivibr2      ; if p6 positive vibrato gets faster
goto vibrato2
vibrato1:
ivibr3          =       giseed
giseed          =       frac(giseed*105.947)
kvibrate        linseg  3.5+ivibr1, .1, 4.5+ivibr2, p3-.1, 2.5+ivibr3   ; if p6 negative vibrato gets slower
vibrato2:
kvibrater       randi   .1*kvibrate, 5, giseed          ; up to 10% vibrato rate variation
giseed          =       frac(giseed*105.947)
kvibrate        =       kvibrate + kvibrater
kvib            oscil   kvibdepth, kvibrate, giwtsin

ifdev1  =       .015 * giseed                           ; frequency deviation
giseed  =       frac(giseed*105.947)
ifdev2  =       -.005 * giseed
giseed  =       frac(giseed*105.947)
ifdev3  =       .003 * giseed
giseed  =       frac(giseed*105.947)
ifdev4  =       .017 * giseed
giseed  =       frac(giseed*105.947)
kfreqr  linseg  ifdev1, iattack, ifdev2, isustain, ifdev3, idecay, ifdev4
kfreq   =       ifreq * (1 + kfreqr) + kvib

if ifreq <  67.13 goto range1                           ; (cpspch(6.00) + cpspch(6.01))/2
if ifreq <  95.56 goto range2                           ; (cpspch(6.06) + cpspch(6.07))/2
if ifreq <  135.16 goto range3                          ; (cpspch(7.00) + cpspch(7.01))/2
if ifreq <  191.11 goto range4                          ; (cpspch(7.06) + cpspch(7.07))/2
goto range5
                                                        ; wavetable amplitude envelopes
range1:                                                 ; for low range tones
kamp1   linseg  0, iatt, -0.012, iatt, 0.000, iatt, 0.494, iatt,        \  
0.717, iatt, 1.005, iatt, 1.396, isus, 1.838, isus, 1.616, isus,        \
1.579, isus, 1.343, idec, 1.231, idec, 0.958, idec, 0.581, idec,        \
0.000, idec, -0.089, idec, 0
kamp2   linseg  0, iatt, 0.301, iatt, 1.000, iatt, 0.891, iatt,         \
0.753, iatt, 0.499, iatt, 0.333, isus, 0.065, isus, 0.125, isus,        \
0.166, isus, -0.002, idec, 0.023, idec, -0.018, idec, 0.029, idec,      \
0.000, idec, 0.389, idec, 0
kamp3   linseg  0, iatt, -0.009, iatt, 0.000, iatt, -0.166, iatt,       \
-0.298, iatt, -0.446, iatt, -0.837, isus, -0.945, isus, -0.644, isus,   \
-0.573, isus, -0.341, idec, -0.254, idec, 0.047, idec, 0.405, idec,     \
1.000, idec, 0.486, idec, 0
iwt1    =       83                                      ; wavetable numbers
iwt2    =       84
iwt3    =       85
inorm   =       29786.7
goto end

range2:                                                 ; for low mid-range tones 

kamp1   linseg  0, iatt, 0.023, iatt, 0.062, iatt, 0.140, iatt,         \
0.605, iatt, 0.902, iatt, 1.302, isus, 1.355, isus, 1.355, isus,        \
1.202, isus, 0.934, idec, 0.891, idec, 0.654, idec, 0.225, idec,        \
0.000, idec, -0.012, idec, 0
kamp2   linseg  0, iatt, 0.035, iatt, 0.230, iatt, 0.637, iatt, .410,   \
iatt, 0.279, iatt, -.240, isus, -0.259, isus, -0.248, isus, -0.086,     \
isus, -0.090, idec, -0.109, idec, -0.074, idec, -0.037, idec, 0.000,    \
idec, 0.016, idec, 0
kamp3   linseg  0, iatt, -0.070, iatt, -0.144, iatt, -0.167, iatt,      \
-.230, iatt, -.350, iatt, -.500, isus, -0.650, isus, -0.435, isus,      \
-0.083, isus, 0.009, idec, 0.030, idec, 0.485, idec, 1.477, idec,       \
1.000, idec, 0.278, idec, 0
iwt1    =       86
iwt2    =       87
iwt3    =       88
inorm   =       23524.2
goto end

range3:                                                 ; for high mid-range tones 

kamp1   linseg  0, iatt, 0.023, iatt, 0.062, iatt, 0.140, iatt,         \ 
0.605, iatt, 0.902, iatt, 1.302, isus, 1.355, isus, 1.355, isus,        \
1.202, isus, 0.934, idec, 0.891, idec, 0.654, idec, 0.225, idec,        \
0.000, idec, -0.012, idec, 0
kamp2   linseg  0, iatt, 0.035, iatt, 0.230, iatt, 0.637, iatt, .410,   \
iatt, 0.279, iatt, -.240, isus, -0.259, isus, -0.248, isus, -0.086,     \
isus, -0.090, idec, -0.109, idec, -0.074, idec, -0.037, idec, 0.000,    \
idec, 0.016, idec, 0
kamp3   linseg  0, iatt, -0.070, iatt, -0.144, iatt, -0.167, iatt,      \
-.230, iatt, -.350, iatt, -.500, isus, -0.650, isus, -0.435, isus,      \
-0.083, isus, 0.009, idec, 0.030, idec, 0.485, idec, 1.477, idec,       \
1.000, idec, 0.278, idec, 0
iwt1    =       89
iwt2    =       90
iwt3    =       91
inorm   =       19174.7
goto end

range4:                                                 ; for high range tones 

kamp1   linseg  0, iatt, 0.015, iatt, 0.000, iatt, 0.504, iatt,         \
0.294, iatt, 0.334, iatt, 0.000, isus, 0.580, isus, 1.166, isus,        \
1.017, isus, 0.996, idec, 1.015, idec, 0.308, idec, 0.002, idec,        \
0.010, idec, -0.005, idec, 0
kamp2   linseg  0, iatt, 0.115, iatt, 1.000, iatt, 0.567, iatt,         \
0.275, iatt, -0.427, iatt, 0.000, isus, -1.425, isus, -1.222, isus,     \
-0.760, isus, -0.270, idec, -0.026, idec, 1.483, idec, 1.819, idec,     \
0.549, idec, 0.188, idec, 0
kamp3   linseg  0, iatt, -0.002, iatt, 0.000, iatt, -0.020, iatt,       \
0.519, iatt, 0.743, iatt, 1.000, isus, 0.853, isus, 0.365, isus,        \
0.371, isus, 0.110, idec, -0.004, idec, 0.185, idec, 0.011, idec,       \
0.001, idec, 0.003, idec, 0
iwt1    =       92
iwt2    =       93
iwt3    =       94
inorm   =       27370.2
goto end

range5:                                                 ; for high range tones 

kamp1   linseg  0, iatt, 0.015, iatt, 0.000, iatt, 0.504, iatt,         \
0.294, iatt, 0.334, iatt, 0.000, isus, 0.580, isus, 1.166, isus,        \
1.017, isus, 0.996, idec, 1.015, idec, 0.308, idec, 0.002, idec,        \
0.010, idec, -0.005, idec, 0
kamp2   linseg  0, iatt, 0.115, iatt, 1.000, iatt, 0.567, iatt,         \
0.275, iatt, -0.427, iatt, 0.000, isus, -1.425, isus, -1.222, isus,     \
-0.760, isus, -0.270, idec, -0.026, idec, 1.483, idec, 1.819, idec,     \
0.549, idec, 0.188, idec, 0
kamp3   linseg  0, iatt, -0.002, iatt, 0.000, iatt, -0.020, iatt,       \
0.519, iatt, 0.743, iatt, 1.000, isus, 0.853, isus, 0.365, isus,        \
0.371, isus, 0.110, idec, -0.004, idec, 0.185, idec, 0.011, idec,       \
0.001, idec, 0.003, idec, 0
iwt1    =       95
iwt2    =       96
iwt3    =       97
inorm   =       22329.4
goto end

end:
kampr1  randi   .02*kamp1, 10, giseed                   ; up to 2% wavetable amplitude variation
giseed  =       frac(giseed*105.947)
kamp1   =       kamp1 + kampr1
kampr2  randi   .02*kamp2, 10, giseed                   ; up to 2% wavetable amplitude variation
giseed  =       frac(giseed*105.947)
kamp2   =       kamp2 + kampr2
kampr3  randi   .02*kamp3, 10, giseed                   ; up to 2% wavetable amplitude variation
giseed  =       frac(giseed*105.947)
kamp3   =       kamp3 + kampr3

awt1    oscili  kamp1, kfreq, iwt1, iphase              ; wavetable lookup
awt2    oscili  kamp2, kfreq, iwt2, iphase
awt3    oscili  kamp3, kfreq, iwt3, iphase
asig    =       awt1 + awt2 + awt3
asig    =       asig*(iampscale/inorm)
kcut    linseg  0, iattack, ifiltcut, isustain, ifiltcut, idecay, 0     ; lowpass filter for brightness control
afilt   tone    asig, kcut
asig    balance afilt, asig
garev   =       garev + asig
        outs12  asig
        endin

;--------------------------------------------------

instr 194                                                              
; add reverb to global signal garev
;       parameters
;       p4      starting reverb time
;       p5      final reverb time
;       p6      % of reverb relative to source signal

irevtime        = p4                                                    ; set first duration of reverb time
ichngtime       = p5                                                    ; set final duration of reverb time
iring           = 0
iring           = (irevtime > ichngtime ? irevtime : iring)             ; set iring to the starting or ending
iring           = (ichngtime > irevtime ? ichngtime : iring)            ; reverbime, whichever is longer

ireverb         = p6                                                    ; percent for reverberated signal
idur            = p3                                                    ; set the duration at p3, and then
p3              = p3 + iring                                            ; lengthen p3 to include longer reverb time
krevtime        linseg  irevtime, idur, irevtime, iring, ichngtime      ; change duration of reverb time
arev            reverb  garev, krevtime                                 ; add reverb to the global signal
                outs12  ireverb * arev                                  ; output reverberated signal
garev =         0                                                       ; set garev to 0 to prevent feedback
                endin

