;=========================================================================
sr = 32000                  ;  MULTI-SAMPLE MIDI ORCHESTRA
kr = 800                    ;     designed and coded by:
ksmps = 40                  ;         Scott Vercoe, August 1996
nchnls = 2                  ;            copyright 1996, Analog Devices, Inc.
;=========================================================================
gisin    ftgen  1, 0, 4096, 10,  1           ; sine wave for chorus
gisaw    ftgen  2, 0, 4096,  7,  -1, 4096, 1 ; saw wave
givcurv  ftgen  3, 0, 16,    5,  5, 16, 50   ; exponential for velocity
gifcurv  ftgen  4, 0, 16,    5,  3, 16, 60   ; exponential for filter
;=========================================================================
gacho init 0   ; CHORUS INIT
turnon 129     ; ENABLE CHORUS
;=========================================================================
givib1  ftload  "12A4.aif"    ; load wavetables
givib2  ftload  "12A5.aif"
iapno1  ftgen  4.35, 0, 0, 1, "prpno1.aif", 0, 0, 0
iapno2  ftgen  5.35, 0, 0, 1, "prpno2.aif", 0, 0, 0
iapno3  ftgen  6.35, 0, 0, 1, "prpno3.aif", 0, 0, 0
iapno4  ftgen  7.35, 0, 0, 1, "prpno4.aif", 0, 0, 0
iapno5  ftgen  8.35, 0, 0, 1, "prpno5.aif", 0, 0, 0
iapno6  ftgen  9.35, 0, 0, 1, "prpno6.aif", 0, 0, 0
iapno7  ftgen 10.35, 0, 0, 1, "prpno7.aif", 0, 0, 0
;=========================================================================
ivibesplt ftsplit 2,6,  0, givib1,    440, ftlptim(givib1), .5, .3,  \
                       78, givib2,    880, ftlptim(givib2), .4, .3
ipnosplit ftsplit 7,6,  0, iapno1,   51.9, ftlptim(iapno1), .6, .3,   \
                       35, iapno2,   87.3, ftlptim(iapno2), .5, .25,  \
                       45, iapno3,  130.8, ftlptim(iapno3), .4, .2,   \
                       52, iapno4,  196,   ftlptim(iapno4), .3, .15,  \
                       59, iapno5,  329.6, ftlptim(iapno5), .3, .15,  \
                       67, iapno6,  523.2, ftlptim(iapno6), .2, .15,  \
                       76, iapno7, 1046.5, ftlptim(iapno7), .15, .3
pgminit  1, ftlptim(givib1),.5, .3, givib1, 440    ; set programs 1 - 7
pgminit  2, ivibesplt
pgminit  3, ivibesplt
pgminit  4, ivibesplt
pgminit  5, ivibesplt
pgminit  6, ivibesplt
pgminit  7, ivibesplt
pgminit  8, ipnosplit
;=========================================================================
; SIMPLE INSTRUMENT ------------------------------------------------------
;=========================================================================
instr   1                                          ; DSP load:  1.570 MIPS
                    vprogs   1                     ; get program 1
        ivel        ampmidi  10000                 ; get direct velocity
        kcps        cpsmidib 2                     ; get pitch
        idl         = v1                           ; lookup: loop delay
        idc         = v2                           ;         decay
        irl         = v3                           ;         release
        ift         = v4                           ;         ftable
        irn         = v5                           ;         root note
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        asig        loscil1  kenv,kcps,ift,irn     ; sample oscillator
                    outs12   asig                  ; outputs
endin
;=========================================================================
; M/S INSTRUMENT ---------------------------------------------------------
;=========================================================================
instr   2                                          ; DSP load:  1.571 MIPS
                    vprogs   2                     ; get program 2
        ivel        ampmidi  10000                 ; get direct velocity
        kcps        cpsmidib 2                     ; get pitch
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        asig        loscil1  kenv,kcps,ift,irn     ; sample oscillator
                    outs12   asig                  ; outputs
endin
;=========================================================================
; M/S INSTRUMENT WITH VELOCITY CURVE -------------------------------------
;=========================================================================
instr   3                                          ; DSP load:  1.573 MIPS
                    vprogs   3                     ; get program 3
        ivel        ampmidi  10000, givcurv        ; get velocity curve
        kcps        cpsmidib 2                     ; get pitch
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        asig        loscil1  kenv,kcps,ift,irn     ; sample oscillator
                    outs12   asig                  ; outputs
endin
;=========================================================================
; M/S INSTRUMENT WITH VELOCITY & FILTER CURVES ---------------------------
;=========================================================================
instr    4                                         ; DSP load:  1.953 MIPS
                    vprogs   4                     ; get program 4
        ivel        ampmidi  10000, givcurv        ; get velocity curve
        iflt        ampmidi  8000,  gifcurv        ; get filter curve
        kcps        cpsmidib 2                     ; get pitch
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        a1          loscil1  kenv,kcps,ift,irn     ; sample oscillator
        asig        butterlp a1, iflt              ; lowpass filter
                    outs12   asig                  ; outputs
endin
;=========================================================================
; M/S INSTRUMENT WITH VIBRATO AND VELOCITY & FILTER CURVES ---------------
;=========================================================================
instr   5                                          ; DSP load:  2.200 MIPS
                    vprogs   5                     ; get program 5
        ivel        ampmidi  10000, givcurv        ; get velocity curve
        iflt        ampmidi  8000,  gifcurv        ; get filter curve
        kcps        cpsmidib 2                     ; get pitch
        kmod        midictrl 1, 0, .02             ; mod wheel
        kvib        oscil    kmod, 4, gisin        ; vibrato LFO
        kcps        = kcps + (kcps*kvib)           ; new frequency
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        a1          loscil1  kenv,kcps,ift,irn     ; sample oscillator
        asig        butterlp a1, iflt              ; lowpass filter
                    outs12   asig                  ; outputs
endin
;=========================================================================
; M/S INSTRUMENT WITH VIBRATO, CHORUS AND VELOCITY & FILTER CURVES -------
;=========================================================================
instr   6                                          ; DSP load:  2.427 MIPS
                    vprogs   6                     ; get program 6
        ivel        ampmidi  10000, givcurv        ; get velocity curve
        iflt        ampmidi  8000,  gifcurv        ; get filter curve
        kcps        cpsmidib 2                     ; get pitch
        kmod        midictrl 1, 0, .01             ; mod wheel
        kvib        oscil    kmod, 4, gisin        ; vibrato LFO
        kcps        = kcps + (kcps*kvib)           ; new frequency
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; envelope
        a1          loscil1  kenv,kcps,ift,irn     ; sample oscillator
        asig        butterlp a1, iflt              ; lowpass filter
                    outs12   asig                  ; outputs
        gacho       addin    asig, .3              ; route to effects buss
endin
;=========================================================================
; M/S INSTRUMENT WITH THE WORKS ------------------------------------------
;=========================================================================
instr   7                                          ; DSP load:  4.409 MIPS
                    vprogs   7                     ; get program 6
        ivel        ampmidi  10000, givcurv        ; get velocity curve
        iflt        ampmidi  8000,  gifcurv        ; get filter curve
        kcps        cpsmidib 2                     ; get pitch
        kmod        midictrl 1, 0, .01             ; mod wheel
        kvib        oscil    kmod, 4, gisin        ; vibrato LFO
        kcps        = kcps + (kcps*kvib)           ; new frequency
ift,irn,idl,idc,irl mtsplit  v1                    ; table lookups
        kenv        dexponr  ivel,idl,idc,irl      ; sample envelope
        a1          loscil1  kenv,kcps,ift,irn     ; sample oscillator
        kenv2       linenr   iflt, 1, .1           ; synth envelope
        asyn        coscil   kenv2,kcps,10,gisaw   ; synth oscillator
        kflt        linsegr  50,2,1000,9,100,5,20,.1,20  ; filter envelope
        asyn        butterlp asyn, kflt            ; synth filter
        asig        butterlp a1, iflt              ; sample filter
        asig        = asig + asyn                  ; combine synth + sample
                    outs12   asig                  ; outputs
        gacho       addin    asig, .3              ; route to effects buss
endin
;=========================================================================
; SIMPLE PIANO INSTRUMENT ------------------------------------------------
;=========================================================================
instr   8
                vprogs   8
                maxalloc 30
        ivel    ampmidi  10000, givcurv         ; get velocity curve
        iflt    ampmidi  8000,  gifcurv         ; get filter curve
        icps    cpsmidi                         ; get pitch
ift,ibsfreq,idel,idec,irel  \                   ; table lookup
                mtsplit  v1
        kpre    dexponr  ivel,idel,idec,irel    ; envelope
        a1      loscil1  kpre,icps,ift,ibsfreq  ; sample oscillator
        asig    butterlp a1, iflt               ; lowpass filter
                outs12   asig                   ; outputs
endin
;=========================================================================
; GLOBAL STEREO CHORUS INSTRUMENT ----------------------------------------
;=========================================================================
        instr       129                            ; DSP load: 16.228 MIPS
acl,acr chorus3     gacho,.191,.217,.233,.425,.001,.003,.005,.007,.1,1,.1, gisin
        outs        acl, acr                       ; stereo outputs
gacho   =           0                              ; reset buss variable
        endin
;=========================================================================
