;===================================================================
; ICMC.ORC: Keyboard Orchestra with Effects and Macros
;===================================================================
; Designed & Coded by:
; Dr. Richard Boulanger and Scotty Vercoe
; for International Computer Music Conference 1996, Hong Kong
; copyright (c) 1996 Analog Devices, Inc.
;===================================================================

macro NOTE          ; ---------------> DEFINE MACROS (1)
    knot    cpsmidib  2              ; Convert MIDI note with Pitchbend
    ivel    ampmidi   8000, givcurv  ; Scale and Map Key Velocity
;            veloffs                  ; Scale Releases by NoteOff Velocity (10)
endm

macro VIBRATO
    kdpth   midictrl  1, 0, .03             ; ModWheel Controls Vibrato Depth
    kvibf   midictrl  2, 1, 8               ; Controller 2 sets Vibrato Speed
    kvib    oscil     kdpth, kvibf, gisin   ; LFO with variable Depth & Speed
    kcpsb   =         knot + (knot * kvib)  ; add LFO to note
endm

macro PARAMETER_CONTROL          ; Maps and Scales Opcode Parameters (11)
    kcutpls midictrl  3, .5,  1  ; Ctrlr#3 sets PulseWave Lowpass Cutoff
    kdetune midictrl  4,  1, 10  ; Ctrlr#4 sets Coscil Detune Ratio
    kwidpls midictrl  5, .1,  1  ; Ctrlr#5 sets Width of Variable Pulse
    kswirl  midictrl  12, .1, 20  ; Ctrlr#6 sets BandPass CntrFreq for Noise
endm

macro MIX_CONTROL
    kmstv   midictrl  7, 0, 1    ; Master Volume Control
    ksmp    midictrl  8, 0, 1    ; Sample Layer Mix Control
    knois   midictrl  9, 0, 1    ; Noise Layer Mix Control
    kpuls   midictrl 10, 0, 1    ; Pulse Layer Mix Control
    kchor   midictrl 11, 0, 1    ; Chorus Layer Mix Control
endm

macro MIX
    amix     =  (asmp * ksmp) + (anois * knois) + (apuls * kpuls) + (achor * kchor)
    asig     =  amix * kmstv
endm

macro PAN                                ; Routes Signal to Output Dacs (16)
    kpan     oscil   .5, .25, gisin      ; Oscillating Pan Function
             panouts asig, kpan + .5     ; Dynamically Panned Stereo Outs
endm

macro SEND                               ; Routes Signal to Global Effects (17)
    gar,gac,gaf maddin  asig, .4, .9, 1  ; Scale, Mix and Send
endm

sr     = 32000  ; ---------> SET SAMPLE/CONTROL RATES & NUMBER OF CHANNELS
kr     =   320
ksmps  =   100
nchnls =     2

gar  init    0  ; ---------> INITIALIZE GLOBAL SENDS
gac  init    0
gaf  init    0

turnon  129  ; ------------> TURN ON GLOBAL REVERB INSTRUMENT (2)
; turnon  130  ;               STEREO CHORUS
; turnon  131  ;               EQ

gisin    ftgen   1, 0, 8192, 10, 1  ; -------> GENERATE FUNCTIONS (3) sine
givcurv  ftgen   2, 0,   16,  5, 5, 16, 50  ;                         exponential

iapno1   ftload  "01F3.aif"   ; -------------> LOAD SAMPLES (4)
iapno2   ftload  "01D#4.aif"
iapno3   ftload  "01C#5.aif"
iapno4   ftload  "01F#5.aif"
iapno5   ftload  "01B5.aif"
iepno1   ftload  "06C4.aif"
iepno2   ftload  "06C5.aif"

; ----------------> SET KEYBOARD SPLITS AND SAMPLE SPECIFIC PARAMETER VALUES (5)
; label   opcode size(5x6) split sample  basfrq   looptime       dec  rel
iapnosplt ftsplit  5,6,     0,   iapno1, 174.6, ftlptim(iapno1), .6, .02,  \
                           58,   iapno2, 311.1, ftlptim(iapno2), .7, .02,  \
                           69,   iapno3, 554.4, ftlptim(iapno3), .8, .025, \
                           77,   iapno4, 740,   ftlptim(iapno4), .8, .025, \
                           82,   iapno5, 987.8, ftlptim(iapno5), .8, .025
iepnosplt ftsplit  2,6,     0,   iepno1, 261.6, ftlptim(iepno1), .3, .007, \
                           67,   iepno2, 523.2, ftlptim(iepno2), .3, .007

; ----------------> MAP SAMPLES TO PROGRAM CHANGE MESSAGES (6)
pgminit 1, iapnosplt  ; read acoustic piano samples with MIDI program #1
pgminit 2, iapnosplt  ;                                  MIDI program #2
pgminit 3, iepnosplt  ; read electric piano samples with MIDI program #3
pgminit 4, iepnosplt  ;                                  MIDI program #4
pgminit 5, iapnosplt
; ---------------> SET INITIAL CONTROLLER VALUES (7)
pctrlinit 1, 1, 80, 2, 30, 3, 70, 4, 20, 5,  34, 12, 64, 7, 100, 8, 100, 9, 10, 10, 20, 11, 19
pctrlinit 2, 1, 45, 2, 94, 3, 76, 4, 19, 5,  90, 12,105, 7, 100, 8,  12, 9, 60, 10, 40, 11, 120
pctrlinit 3, 1,  5, 2, 84, 3, 86, 4, 60, 5, 100, 12, 64, 7, 100, 8, 120, 9,  9, 10, 11, 11, 6
pctrlinit 4, 1, 65, 2, 34, 3, 66, 4, 30, 5,  80, 12, 74, 7, 100, 8,  80, 9, 39, 10, 79, 11, 109
pctrlinit 5, 1, 85, 2, 50, 3, 72, 4,  5, 5, 110, 12, 50, 7, 100, 8,   0, 9,100, 10,  3, 11, 4

; -> EXTENDED CSOUND LAYERED MIDI INSTRUMENT - RICHARD BOULANGER & SCOTTY VERCOE


instr 1                  ; MIPS = 5.937

    vprogs 1, 2, 3, 4, 5 ; Instrument mapped to specific program changes (8)
;   clkon  1             ; Switch on clock to measure DSP load (9)

; ---------------------------------------------> SUBSTITUTE MACROS FROM ABOVE
NOTE
VIBRATO
PARAMETER_CONTROL

; ---------------------------------------------> WAVETABLE LAYER (12)
    ift,ibsfreq,idel,idec,irel  mtsplit  v1    ; Read FTSPLIT Data (from above)
    kpre    dexponr  ivel, idel, idec, irel    ; Loop Sensitive Sample envelope
    asmp    loscil1  kpre, knot, ift, ibsfreq  ; Sample oscillator

; ---------------------------------------------> SWIRLING NOISE LAYER (13)
    a2p     rand     8000
    kswsh   oscil    15, .25, gisin
    a2      butterbp a2p, knot * kswirl, kswsh * 5
    anois   linenr   a2, .6, 0
; ---------------------------------------------> VARIABLE PULSE LAYER (14)
    k3v     linsegr  0, .3, ivel, .3, ivel * .3, .02, .1
    k3ndx   expsegr  .0001, .1, .08, .8,   .3,  .3, .001
    a3p     poscil   k3v, kcpsb, k3ndx * kwidpls
    kswp    linsegr  20, 2, 10000, .002, 100
    apuls   butterlp a3p, kswp * kcutpls

; ---------------------------------------------> CHORUSING SINE WAVE LAYER (15)
    k4v     adsr     ivel, 0, .001, 2, .02, .8, .002, 0
    k4pch   linsegr  4, .001, 1
    achor   coscil   k4v, knot * k4pch, 7 * kdetune, gisin

MIX_CONTROL         ; -------------------------> MACRO SUBSTITUTIONS
MIX
PAN
SEND

; clkoff 1

endin

instr 129  ; ----------------------------------> GLOBAL REVERB INSTRUMENT
    ihirl   =        .1               ; Set High frequency rolloff
    arvb    reverb2  gar, 2.3, ihirl  ; Reverbtime = 2.1 seconds
            outs12   arvb             ; Send signal to Channels 1 and 2
    gar     =        0                ; Clear global variable
endin

instr 130  ; ----------------------------------> GLOBAL STEREO CHORUS INSTRUMENT
    acl,acr chorus2  gac, .09, .31, .53, .72, .001, .002, .003, .005, 0, 1, 0, gisin
            outs     acl, acr
    gac     =        0
endin

instr 131  ; ----------------------------------> GLOBAL EQ INSTRUMENT
    aflt    butterlp gaf, 200    ; Set Lowcut at 200 Hz
    aeq     =        aflt * 2.5  ; Boost Signal at Filter Cutoff Frequency
            outs12   aeq
    gaf     =        0
endin







